local countdowntimer = {}
countdowntimer.init = {}
countdowntimer.maintimer = Timer()
countdowntimer.playsound = gkini.ReadInt("Timer", "sound", 1)
countdowntimer.msg = "Timer has expired"
countdowntimer.start_time = gkini.ReadInt("Timer", "start_time", 0)
countdowntimer.duration = gkini.ReadInt("Timer", "duration", 0)
countdowntimer.version = '1.0'

local SOUND_PATH = "plugins/Timer/Timer_End.ogg"

local sounds = { ['timer_end'] = gksound.GKLoadSound{soundname='timer_end',filename=SOUND_PATH} }

--Check timestamp saved to config file to check if timer needs to be resumed
function countdowntimer.init:PLAYER_ENTERED_GAME(event, data)
	if countdowntimer.start_time > 0 and countdowntimer.duration > 0 then
		local diff_time = os.difftime(os.time(), countdowntimer.start_time)*1000
	
		if diff_time < countdowntimer.duration then
			countdowntimer.maintimer:Kill()
			countdowntimer.maintimer:SetTimeout(countdowntimer.duration-diff_time, countdowntimer.complete)
			local remaining = (countdowntimer.duration-diff_time)/1000
			print('\127ffffffTimer resumed with '..math.floor(remaining/60)..' minutes '..(remaining%60)..' seconds remaining')
		end
	end
end

function countdowntimer:Input(args)
	if args then
		if args[1]:lower() == 'set' then
			if args[2] then
				countdowntimer.maintimer:Kill()
				countdowntimer.duration = countdowntimer.parseTime(args[2])
				countdowntimer.start_time = os.time()
				countdowntimer.maintimer:SetTimeout(countdowntimer.duration, countdowntimer.complete)
				gkini.WriteInt("Timer", "start_time", countdowntimer.start_time)
				gkini.WriteInt("Timer", "duration", countdowntimer.duration)
				print('\127ffffffTimer for '..(countdowntimer.duration/60000)..' minutes has been set')
			else print('\127ffffffYou must specify the timer duration, either HH:MM:SS or just minutes') end
		elseif args[1]:lower() == 'kill' then
			if countdowntimer.maintimer:IsActive() then
				countdowntimer.maintimer:Kill()
				countdowntimer.start_time = 0
				gkini.WriteInt("Timer", "start_time", 0)
				print('\127ffffffTimer aborted')
			else print('\127ffffffTimer has not been set') end
		elseif args[1]:lower() == 'msg' then
			countdowntimer.msg = table.concat(args, " ", 2)
			print('\127ffffffTimer msg set to: '..countdowntimer.msg)
		elseif args[1]:lower() == 'elapsed' then
			if countdowntimer.maintimer:IsActive() then
				local seconds = os.difftime(os.time(), countdowntimer.start_time)
				print('\127ffffffTimer elapsed time: '..math.floor(seconds/60)..' minutes '..(seconds%60)..' seconds')
			else print('\127ffffffTimer has not been set') end
		elseif args[1]:lower() == 'remaining' then
			if countdowntimer.maintimer:IsActive() then
				local seconds = countdowntimer.duration/1000 - os.difftime(os.time(), countdowntimer.start_time)
				print('\127ffffffTimer remaining time: '..math.floor(seconds/60)..' minutes '..(seconds%60)..' seconds')
			else print('\127ffffffTimer has not been set') end
		elseif args[1]:lower() == 'sound' then
			if args[2] then
				if args[2]:lower() == 'on' then
					countdowntimer.playsound = 1
					print('\127ffffffTimer sound playback is ON')
					gkini.WriteInt("Timer", "sound", 1)
				elseif args[2]:lower() == 'off' then
					countdowntimer.playsound = 0
					print('\127ffffffTimer sound playback is OFF')
					gkini.WriteInt("Timer", "sound", 0)
				else print('\127ffffffInvalid argument: use /timer sound on/off') end
			elseif countdowntimer.playsound==1 then
				countdowntimer.playsound = 0
				print('\127ffffffTimer sound playback is OFF')
				gkini.WriteInt("Timer", "sound", 0)
			else
				countdowntimer.playsound = 1
				print('\127ffffffTimer sound playback is ON')
				gkini.WriteInt("Timer", "sound", 1)
			end
		else countdowntimer.help() end
	else countdowntimer.help() end
end

function countdowntimer.parseTime(str)
	local time_tbl = unspickle('\"'..str:gsub(":", "\",\"")..'\"') or {}
	local total_time = 0
	local time_multiplier = 60*60*1000
	
	if table.getn(time_tbl) > 1 then
		for _,t in ipairs(time_tbl) do
			total_time = total_time + tonumber(t)*time_multiplier
			time_multiplier = time_multiplier/60
		end
		
		return total_time
	else return tonumber(str)*60*1000 end
end

function countdowntimer.complete()
	print('\127ffffff'..countdowntimer.msg)
	if countdowntimer.playsound==1 then gksound.GKPlaySound('timer_end') end
end

function countdowntimer.help()
	print('\127ffffffTimer v'..countdowntimer.version..' Commands:'..
	'\n\t/timer set \1270000ffHH\127o:\1270000ffMM\127o:\1270000ffSS\127o - Starts countdown for \1270000ffHH\127o hours, \1270000ffMM\127o minutes, and \1270000ffSS\127o seconds'..
	'\n\t/timer set \1270000ffMM\127o - Starts countdown for \1270000ffMM\127o minutes'..
	'\n\t/timer elapsed OR /timer_elapsed - Display time since timer was activated'..
	'\n\t/timer remaining OR /timer_remaining - Display remaining time until timer expires'..
	'\n\t/timer msg \127ffff00text\127o - Where \127ffff00text\127o is displayed once timer completes'..
	'\n\t/timer kill - aborts timer if active'..
	'\n\t/timer sound on/off - While on plays sound when timer completes')
end

RegisterEvent(countdowntimer.init, "PLAYER_ENTERED_GAME")
RegisterUserCommand("timer", countdowntimer.Input)
RegisterUserCommand("timer_elapsed", function() countdowntimer:Input({'elapsed'}) end)
RegisterUserCommand("timer_remaining", function() countdowntimer:Input({'remaining'}) end)
